@@ -1,13 +1,14 @@
 """
-The pyoperator package contains the following modules or packages:
+The PyOperators package contains the following modules or packages:
 
 - core : defines the Operator class
 - linear : defines standard linear operators
 - nonlinear : defines non-linear operators (such as thresholding or rounding)
-- operators_mpi : MPI operators (even if mpi4py is not present)
 - iterative : defines iterative algorithms working with operators
-
+- utils : miscellaneous routines
+- operators_mpi : MPI operators (even if mpi4py is not present)
 - operators_pywt : (optional) loaded if PyWavelets is present.
+
 """
 
 from .utils import *
@@ -32,4 +33,4 @@ __all__ = [ f for f in dir() if f[0] != '_' and not isinstance(eval(f),
 del f #XXX not necessary with Python3
 del types
 
-__version__ = '0.8.2'
+__version__ = '0.9'
