@@ -13,19 +13,23 @@ from scipy.sparse.linalg import eigsh
 from .decorators import inplace, linear, real, square, symmetric
 from .core import (Operator, BlockRowOperator, BroadcastingOperator,
                    CompositionOperator, DenseOperator, DiagonalOperator,
-                   ReductionOperator, DirectOperatorFactory,
-                   ReverseOperatorFactory, Variable, X, _pool)
+                   HomothetyOperator, ReductionOperator, Variable, X, _pool)
 from .memory import empty
-from .utils import cast, complex_dtype_for, ifirst, izip_broadcast, tointtuple
+from .utils import (
+    cast, complex_dtype, float_dtype, ifirst, izip_broadcast, pi, tointtuple)
 
 __all__ = [
     'BandOperator',
+    'DegreesOperator',
     'DiagonalNumexprOperator',
     'DiagonalNumexprNonSeparableOperator',
     'DifferenceOperator',
     'EigendecompositionOperator',
-    'IntegrationTrapezeWeightOperator',
+    'IntegrationTrapezeOperator',
     'PackOperator',
+    'RadiansOperator',
+    'Rotation2dOperator',
+    'Rotation3dOperator',
     'SumOperator',
     'SymmetricBandOperator',
     'SymmetricBandToeplitzOperator',
@@ -34,6 +38,22 @@ __all__ = [
 ]
 
 
+class DegreesOperator(HomothetyOperator):
+    """
+    Convert angles from radians to degrees.
+
+    Example
+    -------
+    >>> d = DegreesOperator()
+    >>> d(np.pi/2)
+    array(90.0)
+
+    """
+    def __init__(self, dtype=float, **keywords):
+        HomothetyOperator.__init__(self, 180 / pi(dtype), **keywords)
+        self.set_rule('I', lambda s: RadiansOperator(s.dtype))
+
+
 class DiagonalNumexprOperator(DiagonalOperator):
     """
     DiagonalOperator whose diagonal elements are calculated on the fly using
@@ -66,7 +86,7 @@ class DiagonalNumexprOperator(DiagonalOperator):
         self.global_dict = global_dict
         self._global_dict = {} if global_dict is None else global_dict.copy()
         self._global_dict[var] = self.data.T if self.broadcast == \
-                                     'rightward' else self.data
+            'rightward' else self.data
 
     def direct(self, input, output):
         if self.broadcast == 'rightward':
@@ -76,17 +96,18 @@ class DiagonalNumexprOperator(DiagonalOperator):
                          global_dict=self._global_dict, out=output)
 
     def get_data(self):
-        local_dict = {self.var:self.data}
+        local_dict = {self.var: self.data}
         return numexpr.evaluate(self.expr, local_dict=local_dict,
                                 global_dict=self.global_dict)
 
     @staticmethod
-    def _rule_block(self, op, shape, partition, axis, new_axis, func_operation):
+    def _rule_block(self, op, shape, partition, axis, new_axis,
+                    func_operation):
         if type(self) is not DiagonalNumexprOperator:
             return None
-        return DiagonalOperator._rule_block(self, op, shape, partition, axis,
-                   new_axis, func_operation, self.expr, global_dict=
-                   self.global_dict, var=self.var)
+        return DiagonalOperator._rule_block(
+            self, op, shape, partition, axis, new_axis, func_operation,
+            self.expr, global_dict=self.global_dict, var=self.var)
 
 
 class DiagonalNumexprNonSeparableOperator(DiagonalOperator):
@@ -109,17 +130,18 @@ class DiagonalNumexprNonSeparableOperator(DiagonalOperator):
     Example
     -------
     >>> alpha = np.arange(100.)
-    >>> d = DiagonalNumexprNonSeparableOperator('(x/x0)**alpha',
-                                                {'alpha':alpha, 'x':1.2,'x0':1})
+    >>> d = DiagonalNumexprNonSeparableOperator(
+    ...     '(x/x0)**alpha', {'alpha': alpha, 'x': 1.2,'x0': 1})
 
     """
     def __init__(self, expr, global_dict=None, dtype=float, **keywords):
         if not isinstance(expr, str):
             raise TypeError('The first argument is not a string expression.')
         if 'broadcast' in keywords and keywords['broadcast'] == 'rightward':
-            raise ValueError('The class DiagonalNumexprNonSeparableOperator doe'
-                             's not handle rightward broadcasting. Use the clas'
-                             's DiagonalNumexprOperator for this purpose.')
+            raise ValueError(
+                'The class DiagonalNumexprNonSeparableOperator does not handle'
+                ' rightward broadcasting. Use the class DiagonalNumexprOperato'
+                'r for this purpose.')
         if 'broadcast' not in keywords or keywords['broadcast'] != 'leftward':
             keywords['broadcast'] = 'disabled'
         self.expr = expr
@@ -148,134 +170,18 @@ class DiagonalNumexprNonSeparableOperator(DiagonalOperator):
         return None
 
 
-@linear
-class EinsteinSumOperator(Operator):
+@real
+class IntegrationTrapezeOperator(BlockRowOperator):
     """
-    Einstein sum operator.
+    Return a block row operator whose blocks are the weights to perform
+    a trapeze integration.
 
-    Example
-    -------
-    >>> op = EinsteinSumOperator('ii->i', X, shapein=(4,4))
-    >>> op(np.arange(16).reshape(4,4))
-    [0., 5., 10., 15.]
-
-    """
-    def __init__(self, subscripts, *operands, **keywords):
-        subscripts = subscripts.replace(' ', '')
-        if '->' not in subscripts:
-            raise NotImplementedError('Unlike einsum, the output subscript cann'
-                                      'ot yet be specified.')
-        in_, out = subscripts.split('->')
-        ins = in_.split(',')
-        if len(operands) == len(ins) - 1 and X not in operands:
-            operands = operands + (X,)
-        if len(ins) != len(operands):
-            raise ValueError("The number of input operands '{0}' does not match"
-                             " that from the subscripts '{1}'.".format(len(
-                             operands), len(ins)))
-        self.iX = ifirst(operands, lambda x: x is X)
-        self.operands = operands
-        self.subscripts = subscripts
-        self.lsubscripts = tuple([self._get_subscript_as_list(s)
-                                  for s in ins + [out]])
-        Operator.__init__(self, **keywords)
-        tsubscripts = self._get_reversed_subscripts()
-        if self.subscripts == '...jk,...k->...j':
-            tsubscripts = '...kj,...k->...j'
-        elif self.subscripts == '...kj,...k->...j':
-            tsubscripts = '...jk,...k->...j'
-        else:
-            raise NotImplementedError()
-        self.set_rule('.T', lambda s: ReverseOperatorFactory(
-            EinsteinSumOperator, s, tsubscripts, *self.operands))
-
-    def direct(self, input, output):
-        operands = list(self.operands)
-        operands[self.iX] = input
-        np.einsum(self.subscripts, *operands, out=output)
-
-    def reshapein(self, shapein):
-        shapes = [None if isinstance(o, Variable) else o.shape
-                  for o in self.operands] + [None]
-        shapes[self.iX] = shapein
-        return self._get_shape(shapes, -1)
-
-    def reshapeout(self, shapeout):
-        shapes = [None if isinstance(o, Variable) else o.shape
-                  for o in self.operands] + [shapeout]
-        return self._get_shape(shapes, self.iX)
-
-    @staticmethod
-    def _enumerate_ellipsis(l):
-        """
-        Enumerate a sequence that may include an Ellipsis, in which case
-        a slice is returned and negative indices afterwards.
-
-        Example
-        -------
-        >>> list(loop_ellipsis(['a','b',Ellipsis,'c']))
-        [0, 1, slice(2, -1, None), -1]
-
-        """
-        i = 0
-        for x in l:
-            if x is not Ellipsis:
-                yield i, l[i]
-                i += 1
-            else:
-                yield slice(i, i - len(l) + 1), Ellipsis
-                i -= len(l) - 1
-
-    def _get_reversed_subscripts(self):
-        ls = self.lsubscripts
-        ls_reversed = [l[::-1] for l in ls[-2::-1]] + [ls[-1][::-1]]
-        ls_reversed = [''.join(['...' if a is Ellipsis else a for a in l])
-                       for l in ls_reversed]
-        return ','.join(ls_reversed[:-1]) + '->' + ls_reversed[-1]
-
-    def _get_shape(self, shapes, i):
-        """ Infer ith shape from the shapes of the other operands. """
-        shape = []
-        #print 'shapes', shapes
-        for iaxis, subscript in self._enumerate_ellipsis(self.lsubscripts[i]):
-            # find this subscript elsewhere
-            #print 'iaxis, subscript',  iaxis, subscript
-            try:
-                for iother, subscripts_other in enumerate(self.lsubscripts):
-                    #print 'iother, sub_other', iother, subscripts_other, 'skipped' if iother == i or shapes[iother] is None else 'looking...'
-                    if iother == i or shapes[iother] is None:
-                        continue
-                    for iaxis_other, subscript_other in \
-                        self._enumerate_ellipsis(subscripts_other):
-                        #print 'iaxis_other, subscript_other:', iaxis_other, subscript_other
-                        if subscript == subscript_other:
-                            new_dims = tointtuple(shapes[iother][iaxis_other])
-                            shape.extend(new_dims)
-                            raise StopIteration()
-                if i == -1:
-                    raise ValueError("Einstein sum subscripts include an output"
-                                     " subscript '{0}' which never appeared in "
-                                     "an input.".format(subscript))
-                return None
-            except StopIteration:
-                pass
-        return tuple(shape)
-        
-    def _get_subscript_as_list(self, s):
-        s = s.replace('...', '.')
-        return [l if l != '.' else Ellipsis for l in s]
-        
-
-class IntegrationTrapezeWeightOperator(BlockRowOperator):
-    """
-    Return weights as a block row operator to perform trapeze integration.
-
-    This operator can be used to integrate over X the bivariate function 
+    This operator can be used to integrate over X the bivariate function
         f = f(X,Y).
     Let's assume f is sampled at n abscissa x_n non necessarily equally spaced
         f_i(Y) = f(x_i, Y).
-    The operator IntegrationTrapezeWeightOperator returns a block row operator
-        W = [ w_1 * I ... w_n * I]
+    The operator IntegrationTrapezeOperator returns a block row operator
+        W = [ w_1 I ... w_n I]
     such that, given the block column operator
             [ f_1 ]
         F = [ ... ]
@@ -287,15 +193,17 @@ class IntegrationTrapezeWeightOperator(BlockRowOperator):
 
     Example
     -------
+    >>> from pyoperators import BlockColumnOperator
     >>> f = np.power
     >>> x = [0.5,1,2,4]
-    >>> F = BlockColumnOperator([Operator(lambda i,o,v=v:f(v,i,o),
-    ...                         flags='square') for v in x], new_axisout=0)
-    >>> W = IntegrationTrapezeWeightOperator(x)
+    >>> F = BlockColumnOperator(
+    ...         [Operator(lambda i, o, v=v: f(v, i, o), flags='square')
+    ...          for v in x], new_axisout=0)
+    >>> W = IntegrationTrapezeOperator(x)
     >>> int_f = W * F
     >>> int_f([0,1,2])
     array([  3.5   ,   7.875 ,  22.8125])
-    >>> [ trapz(f(x,a), x) for a in [0,1,2] ]
+    >>> [np.trapz(f(x, a), x) for a in [0, 1, 2]]
     [3.5, 7.875, 22.8125]
 
     """
@@ -314,8 +222,8 @@ class IntegrationTrapezeWeightOperator(BlockRowOperator):
                                   **keywords)
 
 
-@linear
 @real
+@linear
 @inplace
 class PackOperator(Operator):
     """
@@ -327,15 +235,15 @@ class PackOperator(Operator):
         Operator.__init__(self, shapein=self.mask.shape,
                           shapeout=np.sum(self.mask), **keywords)
         #XXX .T does not share the same mask...
-        self.set_rule('.T', lambda s: UnpackOperator(~s.mask, dtype=s.dtype))
-        self.set_rule('.T.', '1', CompositionOperator)
+        self.set_rule('T', lambda s: UnpackOperator(~s.mask, dtype=s.dtype))
+        self.set_rule('T,.', '1', CompositionOperator)
 
     def direct(self, input, output):
         output[...] = input[self.mask]
 
 
-@linear
 @real
+@linear
 class UnpackOperator(Operator):
     """
     Convert a vector into an ndarray, under the control of a mask.
@@ -345,14 +253,269 @@ class UnpackOperator(Operator):
         self.mask = ~np.array(mask, np.bool8, copy=False)
         Operator.__init__(self, shapein=np.sum(self.mask),
                           shapeout=self.mask.shape, **keywords)
-        self.set_rule('.T', lambda s: PackOperator(~s.mask, dtype=s.dtype))
-        self.set_rule('.T.', '1', CompositionOperator)
+        self.set_rule('T', lambda s: PackOperator(~s.mask, dtype=s.dtype))
+        self.set_rule('T,.', '1', CompositionOperator)
 
     def direct(self, input, output):
         output[...] = 0
         output[self.mask] = input
 
 
+class RadiansOperator(HomothetyOperator):
+    """
+    Convert angles from degrees to radians.
+
+    Example
+    -------
+    >>> r = RadiansOperator()
+    >>> r(180)
+    array(3.141592653589793)
+
+    """
+    def __init__(self, dtype=float, **keywords):
+        HomothetyOperator.__init__(self, pi(dtype) / 180, **keywords)
+        self.set_rule('I', lambda s: DegreesOperator(s.dtype))
+
+
+@real
+class Rotation2dOperator(DenseOperator):
+    """
+    2-d rotation operator.
+
+    Parameters
+    ----------
+    angle : float, array-like
+        The counter-clockwise rotation angle, in radians.
+    degrees : bool, optional
+        If set, the angle input is in degrees, instead of radians.
+
+    Example
+    -------
+    >>> r = Rotation2dOperator([45, 90], degrees=True)
+    >>> r([1, 0])
+    >>> print r([1, 0])
+    [[  7.07106781e-01   7.07106781e-01]
+     [  6.12323400e-17   1.00000000e+00]]
+
+    """
+    def __init__(self, angle, degrees=False, roll_input=False, dtype=None,
+                 **keywords):
+        angle = np.asarray(angle)
+        if dtype is None:
+            dtype = float_dtype(angle.dtype)
+        if degrees:
+            angle = np.radians(angle)
+        cosa = np.cos(angle)
+        sina = np.sin(angle)
+        m = np.array([[cosa, -sina], [sina, cosa]], dtype=dtype)
+        for i in range(angle.ndim):
+            m = np.rollaxis(m, -1)
+        keywords['flags'] = self.validate_flags(
+            keywords.get('flags', {}), orthogonal=m.ndim == 2)
+        DenseOperator.__init__(self, m, roll_input=roll_input, **keywords)
+
+
+@real
+class Rotation3dOperator(DenseOperator):
+    """
+    Operator for 3-d active rotations about 1, 2 or 3 axes.
+
+    The rotation axes are specified one by one by selecting a convention.
+
+    For intrinsic rotations (in which the coordinate system changes with
+    the rotation), the following conventions are possible:
+        X, Y, Z,
+        XY', XZ', YX', YZ', ZX', ZY',
+        XZ'X'', XZ'Y'', XY'X'', XY'Z'',  YX'Y'', YX'Z'',
+        YZ'Y'', YZ'X'', ZY'Z'', ZY'X'', ZX'Z'' and ZX'Y''.
+    The primes denote the rotated axes after the first elemental rotation and
+    the double primes the rotated axes after the second one.
+
+    And for the extrinsic rotations (in which the original coordinate system
+    remains motionless):
+        X, Y, Z,
+        XY, XZ, YX, YZ, ZX, ZY,
+        XZX, XZY, XYX, XYZ, YXY, YXZ, YZY, YZX, ZYZ, ZYX, ZXZ and ZXY.
+
+    Parameters
+    ----------
+    convention : string
+        Specify from left to right the axes about which the elemental rotations
+        are performed.
+    a1 : float, array-like
+        Rotation angle about the first axis, in radians.
+    a2 : float, array-like
+        Rotation angle about the second axis, in radians.
+    a3 : float, array-like
+        Rotation angle about the third axis, in radians.
+    degrees : bool, optional
+        If set, the angle inputs are in degrees, instead of radians.
+
+    Example
+    -------
+    >>> r1 = Rotation3dOperator("Y", 90, degrees=True)
+    >>> print r1([1, 0, 0])
+    [  6.12323400e-17   0.00000000e+00  -1.00000000e+00]
+    >>> r2 = Rotation3dOperator("XYZ", 30, 40, 50, degrees=True)
+    >>> print r2([1, 0, 0])
+    [ 0.49240388  0.58682409 -0.64278761]d
+    >>> r3 = Rotation3dOperator("ZY'X''", 50, 40, 30, degrees=True)
+    >>> print r3([1, 0, 0])
+    [ 0.49240388  0.58682409 -0.64278761]
+
+    """
+    def __init__(self, convention, a1, a2=0, a3=0, degrees=False,
+                 roll_input=False, dtype=None, **keywords):
+        if not isinstance(convention, str):
+            raise TypeError('Invalid type for the input convention.')
+        convention = convention.upper()
+        if any(l not in "XYZ'" for l in convention):
+            raise ValueError("Invalid convention '{0}'.".format(convention))
+        a1 = np.asarray(a1)
+        a2 = np.asarray(a2)
+        a3 = np.asarray(a3)
+        if dtype is None:
+            dtype = np.find_common_type([float_dtype(a.dtype)
+                                         for a in (a1, a2, a3)], [])
+        a1 = a1.astype(dtype)
+        a2 = a2.astype(dtype)
+        a3 = a3.astype(dtype)
+        if degrees:
+            a1 = np.radians(a1)
+            a2 = np.radians(a2)
+            a3 = np.radians(a3)
+        convention = convention.upper()
+        naxes = len(convention.replace("'", ''))
+
+        # Extrinsic to intrinsic rotation conversion
+        if naxes == 2 and len(convention) == 2:
+            convention = convention[1] + convention[0] + "'"
+            a1, a2 = a2, a1
+        elif naxes == 3 and len(convention) == 3:
+            convention = convention[2:0:-1] + "'" + convention[0] + "''"
+            a1, a3 = a3, a1
+
+        c1 = np.cos(a1)
+        s1 = np.sin(a1)
+        c2 = np.cos(a2)
+        s2 = np.sin(a2)
+        c3 = np.cos(a3)
+        s3 = np.sin(a3)
+        gm = lambda *args: self._get_matrix(*(args + (dtype,)))
+        if convention == 'X':
+            m = gm(1,  0,  0,
+                   0, c1,-s1,
+                   0, s1, c1)
+        elif convention == 'Y':
+            m = gm( c1, 0, s1,
+                     0, 1,  0,
+                   -s1, 0, c1)
+        elif convention == 'Z':
+            m = gm(c1,-s1, 0,
+                   s1, c1, 0,
+                    0,  0, 1)
+        elif convention == "XZ'":
+            m = gm(c2, -s2, 0,
+                   c1*s2, c1*c2, -s1,
+                   s1*s2, c2*s1, c1)
+        elif convention == "XY'":
+            m = gm(c2, 0, s2,
+                   s1*s2, c1, -c2*s1,
+                   -c1*s2, s1, c1*c2)
+        elif convention == "YX'":
+            m = gm(c1, s1*s2, c2*s1,
+                   0, c2, -s2,
+                   -s1, c1*s2, c1*c2)
+        elif convention == "YZ'":
+            m = gm(c1*c2, -c1*s2, s1,
+                   s2, c2, 0,
+                   -c2*s1, s1*s2, c1)
+        elif convention == "ZY'":
+            m = gm(c1*c2,  -s1, c1*s2,
+                   c2*s1, c1, s1*s2,
+                   -s2, 0, c2)
+        elif convention == "ZX'":
+            m = gm(c1, -c2*s1, s1*s2,
+                   s1, c1*c2, -c1*s2,
+                   0, s2, c2)
+        elif convention == "XZ'X''":
+            m = gm(c2, -c3*s2, s2*s3,
+                   c1*s2, c1*c2*c3 - s1*s3,  -c3*s1 - c1*c2*s3,
+                   s1*s2, c1*s3 + c2*c3*s1, c1*c3 - c2*s1*s3)
+        elif convention == "XZ'Y''":
+            m = gm(c2*c3, -s2, c2*s3,
+                   s1*s3 + c1*c3*s2, c1*c2, c1*s2*s3 - c3*s1,
+                   c3*s1*s2 - c1*s3, c2*s1, c1*c3 + s1*s2*s3)
+        elif convention == "XY'X''":
+            m = gm(c2, s2*s3, c3*s2,
+                   s1*s2, c1*c3 - c2*s1*s3, -c1*s3 - c2*c3*s1,
+                   -c1*s2, c3*s1 + c1*c2*s3, c1*c2*c3 - s1*s3)
+        elif convention == "XY'Z''":
+            m = gm(c2*c3, -c2*s3, s2,
+                   c1*s3 + c3*s1*s2, c1*c3 - s1*s2*s3, -c2*s1,
+                   s1*s3 - c1*c3*s2, c3*s1 + c1*s2*s3, c1*c2)
+        elif convention == "YX'Y''":
+            m = gm(c1*c3 - c2*s1*s3, s1*s2, c1*s3 + c2*c3*s1,
+                   s2*s3, c2, -c3*s2,
+                   -c3*s1 - c1*c2*s3, c1*s2, c1*c2*c3 - s1*s3)
+        elif convention == "YX'Z''":
+            m = gm(c1*c3 + s1*s2*s3, c3*s1*s2 - c1*s3, c2*s1,
+                   c2*s3, c2*c3, -s2,
+                   c1*s2*s3 - c3*s1, s1*s3 + c1*c3*s2, c1*c2)
+        elif convention == "YZ'Y''":
+            m = gm(c1*c2*c3 - s1*s3, - c1*s2, c3*s1 + c1*c2*s3,
+                   c3*s2, c2, s2*s3,
+                   -c1*s3 - c2*c3*s1, s1*s2, c1*c3 - c2*s1*s3)
+        elif convention == "YZ'X''":
+            m = gm(c1*c2, s1*s3 - c1*c3*s2, c3*s1 + c1*s2*s3,
+                   s2, c2*c3, -c2*s3,
+                   -c2*s1, c1*s3 + c3*s1*s2, c1*c3 - s1*s2*s3)
+        elif convention == "ZY'Z''":
+            m = gm(c1*c2*c3 - s1*s3,  -c3*s1 - c1*c2*s3, c1*s2,
+                   c1*s3 + c2*c3*s1, c1*c3 - c2*s1*s3, s1*s2,
+                   -c3*s2, s2*s3, c2)
+        elif convention == "ZY'X''":
+            m = gm(c1*c2, c1*s2*s3 - c3*s1, s1*s3 + c1*c3*s2,
+                   c2*s1, c1*c3 + s1*s2*s3, c3*s1*s2 - c1*s3,
+                   -s2, c2*s3, c2*c3)
+        elif convention == "ZX'Z''":
+            m = gm(c1*c3 - c2*s1*s3, -c1*s3 - c2*c3*s1, s1*s2,
+                   c3*s1 + c1*c2*s3, c1*c2*c3 - s1*s3, - c1*s2,
+                   s2*s3, c3*s2, c2)
+        elif convention == "ZX'Y''":
+            m = gm(c1*c3 - s1*s2*s3, -c2*s1, c1*s3 + c3*s1*s2,
+                   c3*s1 + c1*s2*s3, c1*c2, s1*s3 - c1*c3*s2,
+                   -c2*s3, s2, c2*c3)
+        else:
+            raise ValueError(
+                "Invalid rotation convention {0}.".format(convention))
+
+        for i in range(a1.ndim):
+            m = np.rollaxis(m, -1)
+        keywords['flags'] = self.validate_flags(
+            keywords.get('flags', {}), orthogonal=m.ndim == 2)
+        DenseOperator.__init__(self, m, roll_input=roll_input, **keywords)
+
+    @staticmethod
+    def _get_matrix(a11, a12, a13,
+                    a21, a22, a23,
+                    a31, a32, a33, dtype):
+        a11, a12, a13, a21, a22, a23, a31, a32, a33 = np.broadcast_arrays(
+            a11, a12, a13, a21, a22, a23, a31, a32, a33)
+        m = np.empty(a11.shape + (3, 3), dtype)
+        m[..., 0, 0] = a11
+        m[..., 0, 1] = a12
+        m[..., 0, 2] = a13
+        m[..., 1, 0] = a21
+        m[..., 1, 1] = a22
+        m[..., 1, 2] = a23
+        m[..., 2, 0] = a31
+        m[..., 2, 1] = a32
+        m[..., 2, 2] = a33
+        return m
+
+
+@real
 @linear
 class SumOperator(ReductionOperator):
     """
@@ -379,7 +542,7 @@ class SumOperator(ReductionOperator):
 
     """
     def __init__(self, axis=None, dtype=None, skipna=True, **keywords):
-        if np.__version__ < '1.8':
+        if np.__version__ < '2':
             func = np.nansum if skipna else np.add
         else:
             func = np.add
@@ -443,12 +606,13 @@ class TridiagonalOperator(Operator):
 
         shapein = diagonal.size
         if subdiagonal.size not in (1, shapein - 1):
-            raise ValueError('The sub diagonal should be the length of the diag'
-                             'onal minus one or a scalar.')
+            raise ValueError(
+                'The sub diagonal should be the length of the diagonal minus o'
+                'ne or a scalar.')
         if superdiagonal is not None and \
            superdiagonal.size not in (1, shapein - 1):
-            raise ValueError('The super diagonal should be the length of the di'
-                             'agonal minus one or a scalar.')
+            raise ValueError('The super diagonal should be the length of the d'
+                             'iagonal minus one or a scalar.')
 
         if superdiagonal is None:
             superdiagonal = subdiagonal.conj()
@@ -458,21 +622,21 @@ class TridiagonalOperator(Operator):
         self.superdiagonal = superdiagonal
 
         flags = {'real': dtype.kind != 'c',
-                 'symmetric':np.allclose(self.subdiagonal, self.superdiagonal),
-                 'hermitian':np.allclose(self.diagonal.imag, 0) and np.allclose(
-                             self.subdiagonal, self.superdiagonal.conj())}
+                 'symmetric': np.allclose(self.subdiagonal,
+                                          self.superdiagonal),
+                 'hermitian': np.allclose(self.diagonal.imag, 0) and
+                              np.allclose(self.subdiagonal,
+                                          self.superdiagonal.conj())}
         keywords['flags'] = flags
         keywords['shapein'] = shapein
 
         Operator.__init__(self, dtype=dtype, **keywords)
-        self.set_rule('.T', lambda s: DirectOperatorFactory(TridiagonalOperator,
-                      s, s.diagonal, s.superdiagonal, s.subdiagonal))
-        self.set_rule('.C', lambda s: DirectOperatorFactory(TridiagonalOperator,
-                      s, s.diagonal.conj(), s.subdiagonal.conj(),
-                      s.superdiagonal.conj()))
-        self.set_rule('.H', lambda s: DirectOperatorFactory(TridiagonalOperator,
-                      s, s.diagonal.conj(), s.superdiagonal.conj(),
-                      s.subdiagonal.conj()))
+        self.set_rule('T', lambda s: TridiagonalOperator(
+            s.diagonal, s.superdiagonal, s.subdiagonal))
+        self.set_rule('C', lambda s: TridiagonalOperator(
+            s.diagonal.conj(), s.subdiagonal.conj(), s.superdiagonal.conj()))
+        self.set_rule('H', lambda s: TridiagonalOperator(
+            s.diagonal.conj(), s.superdiagonal.conj(), s.subdiagonal.conj()))
 
     def direct(self, input, output):
         output[:] = self.diagonal * input
@@ -503,16 +667,15 @@ class TridiagonalOperator(Operator):
             diags = (self.subdiagonal, self.diagonal, self.superdiagonal)
             for i, d in zip((-1, 0, 1), diags):
                 ab[_band_diag(ku, i)] = d
-            return DirectOperatorFactory(BandOperator, self, ab, kl, ku)
+            return BandOperator(ab, kl, ku)
         else:
-            u = 2 # tridiagonal
+            u = 2  # tridiagonal
             n = self.shape[0]
             # convert to ab format (lower)
             ab = np.zeros((u, n), self.dtype)
             ab[0] = self.diagonal
             ab[1, :-1] = self.subdiagonal
-            return DirectOperatorFactory(SymmetricBandOperator, self, ab,
-                                         lower=True)
+            return SymmetricBandOperator(ab, lower=True)
 
 
 @linear
@@ -522,7 +685,7 @@ class BandOperator(Operator):
     Store a band matrix in ab format as defined in LAPACK
     documentation.
 
-    TODO: direct and transpose methods should call BLAS2 gbmv (not yet in scipy)
+    TODO:direct and transpose methods should call BLAS2 gbmv (not yet in scipy)
     =====
 
     a[i, j] is stored in ab[ku + 1 + i - j, j]
@@ -619,6 +782,7 @@ class BandOperator(Operator):
             rab[_band_diag(rku, -i)] = self.diag(i)
         return rab
 
+
 def _band_diag(ku, i=0):
     """
     Return a slice to get the i-th line of a band operator
@@ -713,9 +877,8 @@ class SymmetricBandOperator(Operator):
         """
         from scipy.linalg import cholesky_banded
 
-        ab_chol = cholesky_banded(self.ab,
-                               overwrite_ab=overwrite_ab,
-                               lower=self.lower)
+        ab_chol = cholesky_banded(self.ab, overwrite_ab=overwrite_ab,
+                                  lower=self.lower)
         if self.lower:
             out = LowerTriangularOperator(self.shape, ab_chol, **self.kwargs)
         else:
@@ -766,7 +929,7 @@ class SymmetricBandToeplitzOperator(Operator):
             fftsize *= 2
         with _pool.get(fftsize, dtype, aligned=True,
                        contiguous=True) as rbuffer:
-            with _pool.get(fftsize // 2 + 1, complex_dtype_for(dtype),
+            with _pool.get(fftsize // 2 + 1, complex_dtype(dtype),
                            aligned=True, contiguous=True) as cbuffer:
                 fplan = pyfftw.FFTW(
                     rbuffer, cbuffer, fftw_flags=[fftw_flag], threads=nthreads)
@@ -790,8 +953,8 @@ class SymmetricBandToeplitzOperator(Operator):
         with _pool.get(self.fftsize, self.dtype, aligned=True,
                        contiguous=True) as rbuffer:
             with _pool.get(
-                self.fftsize // 2 + 1, complex_dtype_for(self.dtype),
-                aligned=True, contiguous=True) as cbuffer:
+                    self.fftsize // 2 + 1, complex_dtype(self.dtype),
+                    aligned=True, contiguous=True) as cbuffer:
                 lpad = (self.bandwidth - 1) // 2
                 x = x.reshape((-1, self.nsamples))
                 out = out.reshape((-1, self.nsamples))
@@ -892,11 +1055,11 @@ class EigendecompositionOperator(CompositionOperator):
             w, v = eigsh(A, return_eigenvectors=True, **kwargs)
         W = DiagonalOperator(w)
         V = DenseOperator(v)
-        V.set_rule('.T.', '1', CompositionOperator)
+        V.set_rule('T,.', '1', CompositionOperator)
         self.eigenvalues = w
         self.eigenvectors = v
         CompositionOperator.__init__(self, [V, W, V.T], **kwargs)
-        self.set_rule('.I', lambda s: s ** -1)
+        self.set_rule('I', lambda s: s ** -1)
 
     def det(self):
         """
