@@ -1,16 +1,13 @@
 from __future__ import division
 
 import collections
-import gc
 import itertools
 import multiprocessing
 import numpy as np
 import operator
 import os
-import cPickle as pickle
 import scipy.sparse
 import signal
-import timeit
 import types
 
 from contextlib import contextmanager
@@ -18,11 +15,11 @@ from itertools import izip
 from . import cythonutils as cu
 
 __all__ = ['all_eq',
-           'benchmark',
            'cast',
-           'complex_dtype_for',
+           'complex_dtype',
            'first',
            'first_is_not',
+           'float_dtype',
            'ifirst',
            'ifirst_is_not',
            'inspect_special_values',
@@ -33,12 +30,13 @@ __all__ = ['all_eq',
            'isscalar',
            'izip_broadcast',
            'least_greater_multiple',
-           'memory_usage',
            'merge_none',
            'ndarraywrap',
+           'one',
            'openmp_num_threads',
            'operation_assignment',
            'operation_symbol',
+           'pi',
            'product',
            'renumerate',
            'strelapsed',
@@ -49,7 +47,9 @@ __all__ = ['all_eq',
            'strshape',
            'tointtuple',
            'uninterruptible',
-           'uninterruptible_if']
+           'uninterruptible_if',
+           'zero']
+
 
 def all_eq(a, b):
     """
@@ -92,186 +92,6 @@ def all_eq(a, b):
         return a.func_code is b.func_code
     return a == b
 
-def benchmark(stmt, args=None, keywords=None, ids=None, setup='pass',
-              verbose=True):
-    """
-    Automate the creation of benchmark tables for functions.
-
-    This tool benchmarks a function given a sequence of different arguments,
-    keywords or identifiers. Note that a little overhead is incurred.
-    It returns a dictionary containing the elapsed time and memory usage
-    for each benchmark.
-
-    Parameters
-    ----------
-    stmt : callable or string
-        The function or snippet to be timed. In case of a string, the arguments
-        are matched by the substrings '*args'.
-        Caveat: these must be interpreted correctly through their repr.
-        Otherwise, they should be passed as a string (then both stmt and setup
-        should be a string). See example below.
-    args : sequence of sequences of arguments, optional
-        The different arguments that will be passed to the function.
-    keywords : sequence of dictionaries of keywords, optional
-        The different keywords that will be passed to the function.
-    ids : sequence of string, optional
-        The identifier for each timing. If not provided, it is inferred
-        from the arguments and the keywords.
-    setup : callable or string, optional
-        Initialisation before timing. In case of a string, arguments and
-        keywords are passed with the same mechanism as the stmt argument.
-    verbose : boolean
-        If true, benchmark results are printed while they are executed.
-
-    Example
-    -------
-    >>> def f(dtype, n=10):
-    ...     return np.zeros(n, dtype)
-    >>> b = benchmark(f, [(float,), (int,)], [{'n':10}, {'n':100}],
-    ...               ['n=10', 'n=100'])
-    n=10: 100000 loops, best of 3: 2.12 usec per loop
-    n=100: 100000 loops, best of 3: 2.21 usec per loop
-
-    >>> class A():
-    ...     def __init__(self, n, dtype=float):
-    ...         self.a = 2 * np.ones(n, dtype)
-    ...     def run(self):
-    ...         np.sqrt(self.a)
-    >>> b = benchmark('a.run()', ['1,dtype=int', '10,dtype=float'],
-    ...               setup='from __main__ import A; a=A(*args)')
-
-    Overhead:
-    >>> def f():
-    ...     pass
-    >>> b = benchmark(f)
-    1000000 loops, best of 3: 0.562 usec per loop
-
-    """
-    if not callable(stmt) and not isinstance(stmt, str):
-        raise TypeError('The argument stmt is neither a string nor callable.')
-
-    if not callable(setup) and not isinstance(setup, str):
-        raise TypeError('The argument setup is neither a string nor callable.')
-
-    precision = 3
-    repeat = 3
-
-    class wrapper(object):
-        def __call__(self):
-            stmt(*self.args, **self.keywords)
-
-    w = wrapper()
-    single_test = args is keywords is None
-
-    def default_args():
-        while True:
-            yield ()
-    def default_keywords():
-        while True:
-            yield {}
-    def default_ids():
-        i = 0
-        while True:
-            yield i
-            i += 1
-
-    # ensure args, keywords and ids are iterators
-    if args is None:
-        args = default_args()
-    elif isinstance(args, (list, tuple)):
-        args = iter(args)
-    if keywords is None:
-        keywords = default_keywords()
-    elif isinstance(keywords, (list, tuple)):
-        keywords = iter(keywords)
-    if ids is None:
-        ids = default_ids()
-    elif isinstance(ids, (list, tuple)):
-        ids = iter(ids)
-
-    result = {'time': [],
-              'VmData': [],
-              'VmRSS': [],
-              'VmSize': []}
-
-    while True:
-
-        try:
-            arg = next(args)
-            keyword = next(keywords)
-            id_ = str(next(ids))
-        except StopIteration:
-            break
-
-        if not isinstance(arg, (list, tuple, str)):
-            raise TypeError('The function arguments must be supplied as a sequ'
-                            'ence.')
-        if not isinstance(keyword, dict):
-            raise TypeError('The function keywords must be supplied as a '
-                            'dict.')
-
-        if isinstance(stmt, str) and isinstance(arg, str):
-            while '*args' in stmt:
-                stmt = stmt.replace('*args', arg)
-        if isinstance(setup, str) and isinstance(arg, str):
-            while '*args' in setup:
-                setup = setup.replace('*args', arg)
-
-        if callable(stmt):
-            w.args = arg
-            w.keywords = keyword
-            t = timeit.Timer(w, setup=setup)
-        else:
-            t = timeit.Timer(stmt, setup=setup)
-
-        # determine number so that 0.2 <= total time < 2.0
-        for i in range(10):
-            number = 10**i
-            x = t.timeit(number)
-            if x >= 0.2:
-                break
-
-        # actual runs
-        gc.collect()
-        memory = memory_usage()
-        if number > 1 or x <= 2:
-            r = t.repeat(repeat, number)
-        else:
-            r = t.repeat(repeat-1, number)
-            r = [x] + r
-        best = min(r)
-        memory = memory_usage(since=memory)
-
-        result['time'].append(best / number)
-        result['VmData'].append(memory['VmData'])
-        result['VmRSS'].append(memory['VmRSS'])
-        result['VmSize'].append(memory['VmSize'])
-
-        if verbose:
-            if id_ != '':
-                id_ += ': '
-            print id_ + "%d loops," % number,
-            usec = best * 1e6 / number
-            if usec < 1000:
-                print "best of %d: %.*g us per loop. " % (
-                    repeat, precision, usec),
-            else:
-                msec = usec / 1000
-                if msec < 1000:
-                    print "best of %d: %.*g ms per loop. " % (
-                        repeat, precision, msec),
-                else:
-                    sec = msec / 1000
-                    print "best of %d: %.*g s per loop. " % (
-                        repeat, precision, sec),
-
-            print ', '.join([k + ':' + str(v) + 'MiB'
-                             for k, v in memory.items()])
-        if single_test:
-            break
-
-    return result
-
 
 def cast(arrays, dtype=None, order='c'):
     """
@@ -287,7 +107,7 @@ def cast(arrays, dtype=None, order='c'):
 
     Example
     -------
-    >>> cast([[1.,2.], None, array(2j)])
+    >>> cast([[1., 2.], None, np.array(2j)])
     (array([ 1.+0.j,  2.+0.j]), None, array(2j))
 
     """
@@ -299,25 +119,63 @@ def cast(arrays, dtype=None, order='c'):
               if a is not None else None for a in arrays)
     return tuple(result)
 
-def complex_dtype_for(dtype):
+
+def complex_dtype(dtype):
     """
-    Return the complex dtype associated to a float dtype.
+    Return the complex dtype associated to a numeric dtype.
 
     Parameter
     ---------
     dtype : dtype
-        The float dtype.
+        The input dtype.
 
     Example
     -------
-    >>> complex_dtype_for(np.float32)
+    >>> complex_dtype(int)
+    dtype('complex128')
+    >>> complex_dtype(np.float32)
     dtype('complex64')
+    >>> complex_dtype(np.float64)
+    dtype('complex128')
+
+    """
+    dtype = float_dtype(dtype)
+    if dtype.kind == 'c':
+        return dtype
+    if dtype == np.float16:
+        if not hasattr(np, 'complex32'):
+            return np.dtype(complex)
+    return np.dtype('complex{}'.format(2 * int(dtype.name[5:])))
+
+
+def float_dtype(dtype):
+    """
+    Return the floating dtype associated to a numeric dtype.
+    Unless the input dtype kind is float or complex, the default float dtype
+    is returned.
+
+    Parameter
+    ---------
+    dtype : dtype
+        The input dtype.
+
+    Example
+    -------
+    >>> float_dtype(int)
+    dtype('float64')
+    >>> float_dtype(np.float32)
+    dtype('float32')
+    >>> float_dtype(np.complex256)
+    dtype('complex256')
 
     """
     dtype = np.dtype(dtype)
-    if dtype.kind != 'f':
-        raise TypeError('The input dtype is not a float.')
-    return np.dtype('complex{}'.format(2 * int(dtype.name[5:])))
+    if dtype.kind not in 'biufc':
+        raise TypeError('Non numerical data type.')
+    if dtype.kind in 'iub':
+        return np.dtype(float)
+    return dtype
+
 
 def first(l, f):
     """
@@ -342,6 +200,7 @@ def first(l, f):
     except StopIteration:
         raise ValueError('There is no matching item in the list.')
 
+
 def first_is_not(l, v):
     """
     Return first item in list which is not the specified value.
@@ -362,6 +221,7 @@ def first_is_not(l, v):
     """
     return next((_ for _ in l if _ is not v), v)
 
+
 def ifirst(l, match):
     """
     Return the index of the first item in a list that verifies a certain
@@ -391,6 +251,7 @@ def ifirst(l, match):
     except StopIteration:
         raise ValueError('There is no matching item in the list.')
 
+
 def ifirst_is_not(l, v):
     """
     Return index of first item in list which is not the specified value.
@@ -411,10 +272,11 @@ def ifirst_is_not(l, v):
 
     """
     try:
-        return next((i for i,_ in enumerate(l) if _ is not v))
+        return next((i for i, _ in enumerate(l) if _ is not v))
     except StopIteration:
         raise ValueError('There is no matching item in the list.')
 
+
 def inspect_special_values(x):
     """
     If an array has no other values than -1, 0 and 1, return a tuple consisting
@@ -452,6 +314,7 @@ def inspect_special_values(x):
         return cu.inspect_special_values_complex128(x.astype(np.complex128))
     return 0, 0, 0, True, False
 
+
 @contextmanager
 def interruptible():
     """ Make a block of code interruptible with CTRL-C. """
@@ -460,6 +323,7 @@ def interruptible():
     yield
     signal.signal(signal.SIGINT, signal_old)
 
+
 @contextmanager
 def interruptible_if(condition):
     """ Conditionally make a block of code interruptible with CTRL-C. """
@@ -469,6 +333,7 @@ def interruptible_if(condition):
         with interruptible():
             yield
 
+
 def isalias(array1, array2):
     """
     Return True if the two input arrays point to the same memory location.
@@ -477,6 +342,7 @@ def isalias(array1, array2):
     return array1.__array_interface__['data'][0] == \
            array2.__array_interface__['data'][0]
 
+
 def isclassattr(cls, a):
     """ Test if an attribute is a class attribute. """
     for c in cls.__mro__:
@@ -484,6 +350,7 @@ def isclassattr(cls, a):
             return True
     return False
 
+
 def isscalar(data):
     """Hack around np.isscalar oddity"""
     if isinstance(data, np.ndarray):
@@ -494,6 +361,7 @@ def isscalar(data):
         return False
     return True
 
+
 def izip_broadcast(*args):
     """
     Like izip, except that arguments which are containers of length 1 are
@@ -508,6 +376,7 @@ def izip_broadcast(*args):
         args = [wrap(arg) for arg in args]
     return izip(*args)
 
+
 def least_greater_multiple(a, l, out=None):
     """
     Return the least multiple of values in a list greater than a given number.
@@ -521,8 +390,8 @@ def least_greater_multiple(a, l, out=None):
     if any(v <= 0 for v in l):
         raise ValueError('The list of multiple is not positive;')
     it = np.nditer([a, out],
-                   op_flags = [['readonly'],
-                               ['writeonly', 'allocate', 'no_broadcast']])
+                   op_flags=[['readonly'],
+                             ['writeonly', 'allocate', 'no_broadcast']])
     max_power = [int(np.ceil(np.log(np.max(a))/np.log(v))) for v in l]
     slices = [slice(0, m+1) for m in max_power]
     powers = np.ogrid[slices]
@@ -530,57 +399,16 @@ def least_greater_multiple(a, l, out=None):
     for v, p in izip(l, powers):
         values = values * v**p
     for v, o in it:
-        if np.__version__ >= '1.8':
-            o[...] = np.amin(values, where=values>=v)
-        else:
-            values_ = np.ma.MaskedArray(values, mask=values<v, copy=False)
+        if np.__version__ < '2':
+            values_ = np.ma.MaskedArray(values, mask=values < v, copy=False)
             o[...] = np.min(values_)
+        else:
+            o[...] = np.amin(values, where=values >= v)
     out = it.operands[1]
     if out.ndim == 0:
         return out.flat[0]
     return out
 
-def memory_usage(keys=('VmRSS', 'VmData', 'VmSize'), since=None):
-    """
-    Return a dict containing information about the process' memory usage.
-
-    Parameters
-    ----------
-    keys : sequence of strings
-        Process status identifiers (see /proc/###/status). Default are
-        the resident, data and virtual memory sizes.
-    since : dict
-        Dictionary as returned by a previous call to memory_usage function and
-        used to compute the difference of memory usage since then.
-        
-    """
-    proc_status = '/proc/%d/status' % os.getpid()
-    scale = {'kB': 1024, 'mB': 1024*1024,
-             'KB': 1024, 'MB': 1024*1024}
-
-    # get pseudo file  /proc/<pid>/status
-    with open(proc_status) as f:
-        status = f.read()
-
-    result = {}
-    for k in keys:
-        # get VmKey line e.g. 'VmRSS:  9999  kB\n ...'
-        i = status.index(k)
-        v = status[i:].split(None, 3)  # whitespace
-        if len(v) < 3:
-            raise ValueError('Invalid format.')
-
-        # convert Vm value to Mbytes
-        result[k] = float(v[1]) * scale[v[2]] / 2**20
-
-    if since is not None:
-        if not isinstance(since, dict):
-            raise TypeError('The input is not a dict.')
-        common_keys = set(result.keys())
-        common_keys.intersection_update(since.keys())
-        result = dict((k, result[k] - since[k]) for k in common_keys)
-
-    return result
 
 def merge_none(a, b):
     """
@@ -603,25 +431,34 @@ def merge_none(a, b):
         return None
     if len(a) != len(b):
         raise ValueError('The input sequences do not have the same length.')
-    if any(p != q for p,q in izip(a,b) if None not in (p,q)):
+    if any(p != q for p, q in izip(a, b) if None not in (p, q)):
         raise ValueError('The input sequences have incompatible values.')
-    return tuple(p if p is not None else q for p,q in izip(a,b))
-    
+    return tuple(p if p is not None else q for p, q in izip(a, b))
+
+
 class ndarraywrap(np.ndarray):
     pass
 
+
+def one(dtype):
+    """ Return 1 with a given dtype. """
+    return np.ones((), dtype=dtype)[()]
+
+
 def openmp_num_threads():
     n = os.getenv('OMP_NUM_THREADS')
     if n is not None:
         return int(n)
     return multiprocessing.cpu_count()
-    
+
+
 def operation_assignment(a, b):
     """
     operation_assignment(a, b) -- Same as a[...] = b.
     """
     a[...] = b
 
+
 operation_symbol = {
     operator.iadd: '+',
     operator.isub: '-',
@@ -629,6 +466,12 @@ operation_symbol = {
     operator.idiv: '/',
 }
 
+
+def pi(dtype):
+    """ Return pi with a given dtype. """
+    return 4 * np.arctan(one(dtype))
+
+
 def product(a):
     """ Return the product of a arbitrary input, including generators. """
     if isinstance(a, (list, tuple, types.GeneratorType)):
@@ -641,10 +484,12 @@ def product(a):
     a = np.asarray(a)
     return np.product(a, dtype=a.dtype)
 
+
 def renumerate(l):
     """ Reversed enumerate. """
     return izip(xrange(len(l)-1, -1, -1), reversed(l))
 
+
 def strelapsed(t0, msg='Elapsed time'):
     """
     Return an information message including elapsed time.
@@ -663,11 +508,12 @@ def strelapsed(t0, msg='Elapsed time'):
     >>> pass
     >>> print(strelapsed(t0, 'Did nothing in'))
     Info computernode: Did nothing in... 0.00s
- 
+
     """
     import time
     return strinfo(msg + '... {0:.2f}s'.format(time.time()-t0))[:-1]
 
+
 def strenum(choices, last='or'):
     """
     Enumerates elements of a list
@@ -685,13 +531,14 @@ def strenum(choices, last='or'):
     "'blue', 'red' or 'yellow'"
 
     """
-    choices = [ "'{0}'".format(choice) for choice in choices ]
+    choices = ["'{0}'".format(choice) for choice in choices]
     if len(choices) == 0:
         raise ValueError('There is no valid choice.')
     if len(choices) == 1:
         return choices[0]
     return ', '.join(choices[0:-1]) + ' ' + last + ' ' + choices[-1]
 
+
 def strinfo(msg):
     """
     Return information message adding processor's node name.
@@ -703,12 +550,13 @@ def strinfo(msg):
     Example
     -------
     >>> print(strinfo('My information message'))
-    Info computernode: My information message. 
+    Info computernode: My information message.
 
     """
     import platform
     return 'Info {0}: {1}.'.format(platform.node(), msg)
 
+
 def strnbytes(nbytes):
     """
     Return number of bytes in a human readable unit of KiB, MiB or GiB.
@@ -734,6 +582,7 @@ def strnbytes(nbytes):
     else:
         return str(nbytes / 2**30) + ' GiB'
 
+
 def strplural(n, name, nonumber=False, s=''):
     """
     Returns the plural or singular of a string
@@ -773,6 +622,7 @@ def strplural(n, name, nonumber=False, s=''):
     else:
         return ('' if nonumber else str(n) + ' ') + name + 's' + s
 
+
 def strshape(shape):
     """ Helper function to convert shapes or list of shapes into strings. """
     if shape is None or len(shape) == 0:
@@ -781,7 +631,7 @@ def strshape(shape):
         return ', '.join(strshape(s) for s in shape)
     if len(shape) == 1:
         return str(shape[0])
-    return str(shape).replace(' ','')
+    return str(shape).replace(' ', '')
 
 
 def tointtuple(data):
@@ -793,6 +643,7 @@ def tointtuple(data):
     except TypeError:
         return (int(data),)
 
+
 @contextmanager
 def uninterruptible():
     """
@@ -803,6 +654,7 @@ def uninterruptible():
     signal_old = signal.getsignal(signal.SIGINT)
     #XXX the nonlocal Python3 would be handy here
     ctrlc_is_pressed = []
+
     def signal_handler(signal, frame):
         ctrlc_is_pressed.append(True)
     signal.signal(signal.SIGINT, signal_handler)
@@ -811,6 +663,7 @@ def uninterruptible():
     if len(ctrlc_is_pressed) > 0:
         raise KeyboardInterrupt()
 
+
 @contextmanager
 def uninterruptible_if(condition):
     """ Conditionally make a block of code uninterruptible with CTRL-C. """
@@ -819,3 +672,8 @@ def uninterruptible_if(condition):
     else:
         with uninterruptible():
             yield
+
+
+def zero(dtype):
+    """ Return 0 with a given dtype. """
+    return np.zeros((), dtype=dtype)[()]
