@@ -20,6 +20,7 @@ __all__ = ['assert_eq',
            'skiptest_if',
            'skiptest_unless_module']
 
+
 def assert_same(actual, desired, rtol=2, broadcasting=False):
     """
     Compare arrays of floats. The relative error depends on the data type.
@@ -40,7 +41,10 @@ def assert_same(actual, desired, rtol=2, broadcasting=False):
             "1}'.".format(actual.shape, desired.shape))
     dtype = sorted(arg.dtype for arg in [actual, desired])[0]
     if dtype.kind in ('b', 'i'):
-        assert_equal(actual, desired)
+        if not broadcasting:
+            assert_equal(actual, desired)
+        else:
+            assert np.all(actual == desired)
         return
     eps = np.finfo(dtype).eps * rtol
     same = abs(actual - desired) <= eps * np.maximum(abs(actual), abs(desired))
@@ -55,47 +59,58 @@ def assert_same(actual, desired, rtol=2, broadcasting=False):
             "Arrays are not equal.\n\n(mismatch {0:.1%})\n x: {1}\n y: {2}"
             .format(1 - np.mean(same), trepr(actual), trepr(desired)))
 
+
 def assert_eq(a, b, msg=None):
     """ Assert that the two arguments are (almost) equal. """
     assert all_eq(a, b), msg
 
+
 def assert_in(a, b, msg=None):
     """ Assert that the first argument is in the second one. """
     if a in b:
         return
     assert False, str(a) + ' is not in ' + str(b) + _get_msg(msg)
 
+
 def assert_not_in(a, b, msg=None):
     """ Assert that the first argument is not in second one. """
     if a not in b:
         return
     assert False, str(a) + ' is in ' + str(b) + _get_msg(msg)
 
+
 def assert_is(a, b, msg=None):
     """ Assert arguments are equal as determined by the 'is' operator. """
     if a is b:
         return
     assert False, str(a) + ' is not ' + str(b) + _get_msg(msg)
 
+
 def assert_is_not(a, b, msg=None):
     """ Assert arguments are not equal as determined by the 'is' operator. """
     if a is not b:
         return
     assert False, str(a) + ' is ' + str(b) + _get_msg(msg)
 
+
 def assert_is_instance(a, cls, msg=None):
     """ Assert that the first argument is an instance of the second one. """
     if isinstance(a, cls):
         return
     assert False, str(a) + " is not a '" + cls.__name__ + "' instance" + \
-           _get_msg(msg)
+        _get_msg(msg)
+
 
 def assert_is_not_instance(a, cls, msg=None):
-    """ Assert that the first argument is not an instance of the second one. """
+    """
+    Assert that the first argument is not an instance of the second one.
+
+    """
     if not isinstance(a, cls):
         return
     assert False, str(a) + " is a '" + cls.__name__ + "' instance" + \
-           _get_msg(msg)
+        _get_msg(msg)
+
 
 def assert_is_none(a, msg=None):
     """ Assert argument is None. """
@@ -103,12 +118,14 @@ def assert_is_none(a, msg=None):
         return
     assert False, str(a) + ' is not None' + _get_msg(msg)
 
+
 def assert_is_not_none(a, msg=None):
     """ Assert argument is not None. """
     if a is not None:
         return
     assert False, str(a) + ' is None' + _get_msg(msg)
 
+
 def assert_is_type(a, cls, msg=None):
     """ Assert argument is of a specified type. """
     if type(cls) is type:
@@ -121,16 +138,19 @@ def assert_is_type(a, cls, msg=None):
         "{0} is of type '{1}' instead of {2}{3}".format(
         a, type(a).__name__, strenum(c.__name__ for c in cls), _get_msg(msg)))
 
+
 def assert_raises(*args, **kwargs):
     np.testing.assert_raises(*args, **kwargs)
 assert_raises.__doc__ = np.testing.assert_raises.__doc__
 
+
 def skiptest(func):
     @functools.wraps(func)
     def _():
         raise SkipTest()
     return _
 
+
 def skiptest_if(condition):
     def decorator(func):
         @functools.wraps(func)
@@ -141,6 +161,7 @@ def skiptest_if(condition):
         return _
     return decorator
 
+
 def skiptest_unless_module(module):
     def decorator(func):
         @functools.wraps(func)
@@ -153,6 +174,7 @@ def skiptest_unless_module(module):
         return _
     return decorator
 
+
 def _get_msg(msg):
     if not msg:
         return '.'
